
/********************************************************
** application.c
**
** Part of Calendar Project, Copyright (c) 2004-2005 [Yak]
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
********************************************************/

#include <nokix/all.h>
#include "calendar.h"

#define STATE_CALENDAR        0
#define STATE_MENU            1

/* messages from menu */
#define OMSG_QUIT             (0x1f00)
#define OMSG_OPTIONS          (0x1f01)
#define OMSG_TODAY            (0x1f02)
#define OMSG_SET_DATE         (0x1f03)

/* external data */
extern unsigned char str_date_is_set[];
extern struct menu menu_app;

/* date */
static struct date date;

#define state                 (global_buffer[0])

/********************************************************************************/

/* main system messages dispatcher */
unsigned cal_app_disp(unsigned msg)
{
   switch(msg)
   {
      case MSG_D_INIT:
         if(get_timestamp()!=TIMESTAMP_UNSET)
         {
            if((date.year==0)&&(date.month==0)&&(date.day==0))
            {
               struct datetime dt;

               get_datetime(&dt);
               memcpy(&date,&dt,sizeof(struct date));
            }
    
            calendar_open(NULL);
            calendar_set_date(&date,FALSE);

            state=STATE_CALENDAR;
            msg=MSG_R_STAY;
         }
         else
         {
            /* error: time not set */
            send_message(MSG_2P|MSG_DISPLAY_MESSAGE,DMI_INFORMATION,str_set_clock);
            send_message(MSG_1P|MSG_START_HANDLER,id_set_clock);
         }
         break;

      case MSG_D_QUIT:
         calendar_get_date(&date); // save it for later
         calendar_close();
         msg=MSG_R_QUIT;
         break;

      case MSG_D_QUIT_NOW:
         msg=MSG_R_STAY;
         break;

      case MSG_KEYPRESS:
         if(state==STATE_CALENDAR) calendar_keypress(own_get_unify_keycode(UKM_NPM5_CURSORS1));
         break;

      case MSG_KEYPRESS_ASCII:
         if(state==STATE_CALENDAR) msg=MSG_NOP;
         break;

      case OMSG_OPTIONS:
         send_message(MSG_OPEN_STATIC_MENU|MSG_1P,&menu_app);
         state=STATE_MENU;
         break;

      case MSG_CLOSE_MENU:
         state=STATE_CALENDAR;
         break;

      case MSG_TIMESTAMP:
      case MSG_CONFIG_CHANGED:
         calendar_draw(FALSE);
         break;

      case OMSG_QUIT:
         state=STATE_CALENDAR;
         send_message(MSG_REQUEST_QUIT);
         msg=MSG_NOP;
         break;

      case OMSG_TODAY:
         state=STATE_CALENDAR;
         calendar_keypress(KEY_HASH);
         msg=MSG_NOP;
         break;

      case OMSG_SET_DATE:
         {
            struct datetime dt;
            long stamp=get_timestamp();
   
            if(stamp!=TIMESTAMP_UNSET)
            {
               state=STATE_CALENDAR;
               calendar_get_date((struct date *)&dt);
               memset(&dt.hours,0,sizeof(struct time));
               add_time((struct time *)&dt.hours,stamp);
               set_timestamp(calc_timestamp(&datetime_zero,&dt));
               send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_DONE,&str_date_is_set);
               calendar_draw(false);
            }
            else
            {
               send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,str_set_clock);
               menu_reopen();
            }
            msg=MSG_NOP;
         }
         break;
   }
   return msg;
}

