
/********************************************************
** reminders.c
**
** Part of Calendar Project, Copyright (c) 2004-2005 [Yak]
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
********************************************************/

#include <nokix/all.h>
#include "calendar.h"
/*
#include <stdio.h>

static void dump_dt(struct datetime *dt)
{
   printf("Datetime @ 0x%x\n",(unsigned)dt);
   printf("Date: %02d-%02d-%04d\n",dt->day,dt->month,dt->year);
   printf("Time: %02d:%02d:%02d\n",dt->hours,dt->mins,dt->secs);
}
*/

/* states */
#define STATE_CALENDAR           0
#define STATE_MENU               1

/* global buffer vars */
#define state                    (global_buffer[0])
#define view_mode                (global_buffer[1])

/* own messages (main app) */
#define OMSG_INIT                (0x1f00)
#define OMSG_OPTIONS             (0x1f01)
#define OMSG_REDRAW              (0x1f02)
#define OMSG_SET_DATE            (0x1f03)

/* own messages (view) */
#define OMSG_VIEW_INIT           (0x1f10)

/* reminders table item */
struct reminder
{
   long stamp;
   unsigned char unk0;
   unsigned char unk1;
   unsigned short unk2; /* bits 6-13 (8bits) : index in eeprom */
};

/* external data */
extern struct menu menu_reminders;
extern unsigned char reminders_date[];
extern signed char current_reminder;
extern signed char reminders_count;
extern struct reminder reminders_table[];
extern unsigned char str_no_day_notes[];
extern unsigned char str_date_is_set[];

extern rexx_uint view_reminders_id;
extern rexx_uint add_msg;
extern rexx_uint view_msg;
extern rexx_uint redraw_msg;

/********************************************************************************/

/* this function will be called by main calendar procedures to check if
   a date should be drawn in bold, of course we search the reminders
   table to find this information */
static unsigned reminders_handler(unsigned mode, unsigned *udata, void *arg)
{
   struct date *date = (struct date *) arg;
   struct date *tab;
   int i;

   /* for speed we convert timestamps from reminders table to date structs
      which can be quickly compared without calculations */
   if(mode==CHM_INIT)
   {
      tab=malloc(sizeof(struct date)*reminders_count);
      for(i=0;i<reminders_count;i++)
      {
         memcpy(&tab[i],&datetime_zero,sizeof(struct date));
         add_date(&tab[i],reminders_table[i].stamp);
      }

      *udata=(ULONG)tab;
      return 1;
   }
   /* here we free the converted table */
   else if(mode==CHM_EXIT)
   {
      free((void *)*udata);
      return 1;
   }
   /* date checking */
   else if(mode==CHM_CHECK_DATE)
   {
      tab=(struct date *)*udata;

      for(i=0;i<reminders_count;i++)
         if(!memcmp(date,&tab[i],sizeof(struct date)))
            return true; /* yep, there is a reminder for this day */
      return false; /* nope, no reminders for this day */
   }
   return 0;
}

/********************************************************************************/
/* these functions must be used beacause reminders_date can be on address not
   divisable by four which is required by GCC */

static struct date *reminders_date_get(struct date *date)
{
   memcpy(date,reminders_date,sizeof(struct date));
   return date;
}

static struct date *reminders_date_set(struct date *date)
{
   memcpy(reminders_date,date,sizeof(struct date));
   return date;
}

/********************************************************************************/

/* main app system messages dispatcher */
unsigned cal_reminders_disp(unsigned msg)
{
   struct date rdate;

   if((msg==add_msg)||(msg==view_msg))
   {
      calendar_get_date(&rdate);
      reminders_date_set(&rdate);
      return msg;
   }
   if((msg==redraw_msg)||(msg==0x6f2))
   {
      /* 6F2 is the message that comes when the "Are you sure to delete all
         reminders?" dialog is accepted. This is needed only in models where
         this funcion is placed in the main Reminders dispatcher */
      send_message(OMSG_REDRAW);
      return msg;
   }

   switch(msg)
   {
      case MSG_D_INIT:
         state=STATE_CALENDAR;
         calendar_open(reminders_handler);
         send_message(OMSG_INIT);
         break;

      case OMSG_INIT:
         reminders_date_get(&rdate);
         if(rdate.year==0)
         {
            struct datetime dt;

            get_datetime(&dt);
            reminders_date_set((struct date *)&dt);
         }
         reminders_date_get(&rdate);
         calendar_set_date(&rdate,false);
         msg=MSG_NOP;
         break;

      case MSG_D_QUIT:
         calendar_close();
         break;

      case MSG_KEYPRESS:
         if(state==STATE_CALENDAR)
         {
            unsigned key=own_get_unify_keycode(UKM_NPM5_CURSORS1);

            if(key==KEY_5)
            {
               calendar_get_date(&rdate);
               reminders_date_set(&rdate);
               send_message(MSG_START_HANDLER|MSG_3P,view_reminders_id,0,0);
            }
            else calendar_keypress(key);
         }
         break;

      case MSG_KEYPRESS_ASCII:
         if(state==STATE_CALENDAR) msg=MSG_NOP;
         break;

      case MSG_D_QUIT_NOW:
         if(state==STATE_MENU) menu_reopen();
         send_message(MSG_NUMERIC_MODE);
         msg=MSG_R_STAY;
         reminders_date_get(&rdate);
         calendar_set_date(&rdate,true);
         break;

      case MSG_REQUEST_QUIT:
         if(state!=STATE_CALENDAR) msg=0x5ea;
         break;

      case OMSG_OPTIONS:
         state=STATE_MENU;
         send_message(MSG_OPEN_STATIC_MENU|MSG_1P,&menu_reminders);
         msg=MSG_NOP;
         break;

      case MSG_CLOSE_MENU:
         state=STATE_CALENDAR;
         break;

      case MSG_TIMESTAMP:
      case MSG_CONFIG_CHANGED:
         calendar_draw(false);
         break;

      case OMSG_REDRAW:
         calendar_draw(true);
         msg=MSG_NOP;
         break;

      case OMSG_SET_DATE:
         {
            struct datetime dt;

            get_datetime(&dt);
            calendar_get_date((struct date *)&dt);
            set_timestamp(calc_timestamp(&datetime_zero,&dt));
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_DONE,str_date_is_set);
            calendar_draw(false);
            state=STATE_CALENDAR;
            msg=MSG_NOP;
         }
         break;
   }
   return msg;
}

/********************************************************************************/

/* this routine searches the reminders table to find a reminder for
   specified day; serching starts at <start> and continues upwards */
static long find_reminder(long start, struct date *date)
{
   struct date rdate;
   long i;

   if(reminders_count==0) return -1;

   if(start<1) start=1;
   else if(start>reminders_count) start=reminders_count;

   for(i=start-1;i<reminders_count;i++)
   {
      memcpy(&rdate,&datetime_zero,sizeof(struct date));
      add_date(&rdate,reminders_table[i].stamp);
      if(!memcmp(date,&rdate,sizeof(struct date))) return i+1;
   }
 
   if(start>1) return find_reminder(1,date);
   return -1;
}

/* this routine searches the reminders table to find a reminder for
   specified day; serching starts at <start> and continues downwards */
static long find_reminder_back(long start, struct date *date)
{
   struct date rdate;
   long i;

   if(reminders_count==0) return -1;

   if(start<1) start=1;
   else if(start>reminders_count) start=reminders_count;

   for(i=start-1;i>=0;i--)
   {
      memcpy(&rdate,&datetime_zero,sizeof(struct date));
      add_date(&rdate,reminders_table[i].stamp);
      if(!memcmp(date,&rdate,sizeof(struct date))) return i+1;
   }
 
   if(start<reminders_count) return find_reminder_back(reminders_count,date);
   return -1;
}

/********************************************************************************/

/* view reminders system messages dispatcher (patch) */
unsigned cal_reminders_view_disp(unsigned msg)
{
   struct date rdate;
   
   if(msg==MSG_D_PHONE_STARTUP) return msg;
   else if(msg==MSG_D_INIT)
   {
      view_mode=msg_argv[2];
      if(view_mode==0)
      {
         reminders_date_get(&rdate);
         if(find_reminder(1,&rdate)==-1)
         {
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,str_no_day_notes);
            msg=0x5ea;
         }
         else send_message(OMSG_VIEW_INIT);
      }
   }
   else if(view_mode==0) /* view day notes */
   {
      reminders_date_get(&rdate);
      switch(msg)
      {
         case OMSG_VIEW_INIT:
            current_reminder=find_reminder(1,&rdate);
            msg=0x6e1;
            break;

         case MSG_KEY_UP:
            current_reminder=find_reminder_back(current_reminder-1,&rdate);
            msg=0x6e1;
            break;

         case MSG_KEY_DOWN:
            current_reminder=find_reminder(current_reminder+1,&rdate);
            msg=0x6e1;
            break;

         case MSG_D_QUIT_NOW:
            current_reminder=find_reminder(current_reminder,&rdate);
            if(current_reminder==-1)
            {
               send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,str_no_day_notes);
               msg=0x5ea;
            }
            break;
      }
   }
   return msg;
}

